@@ -14,6 +14,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -45,13 +46,13 @@
 #define IXDR_PUT_U_LONG(buf, v) IXDR_PUT_LONG(buf, (long)(v))
 #endif
 
-#include "list.h"
-#include "locking.h"
-#include "logging.h"
-#include "lkowner.h"
-#include "compat-uuid.h"
-#include "refcount.h"
-#include "atomic.h"
+#include "glusterfs/list.h"
+#include "glusterfs/locking.h"
+#include "glusterfs/logging.h"
+#include "glusterfs/lkowner.h"
+#include "glusterfs/compat-uuid.h"
+#include "glusterfs/refcount.h"
+#include "glusterfs/atomic.h"
 
 #define GF_YES 1
 #define GF_NO 0
@@ -89,7 +90,7 @@
 #define GF_OFF_MAX ((1ULL << (sizeof(off_t) * 8 - 1)) - 1ULL)
 
 #define GLUSTERD_MAX_SNAP_NAME 255
-#define GLUSTERFS_SOCKET_LISTEN_BACKLOG 10
+#define GLUSTERFS_SOCKET_LISTEN_BACKLOG 1024
 
 #define SLEN(str) (sizeof(str) - 1)
 
@@ -178,6 +179,8 @@
 
 #define GLUSTERFS_INTERNAL_FOP_KEY "glusterfs-internal-fop"
 
+#define GF_ENFORCE_MANDATORY_LOCK "trusted.glusterfs.enforce-mandatory-lock"
+
 /* GlusterFS Internal FOP Indicator flags
  * (To pass information on the context in which a paritcular
  *  fop is performed between translators)
@@ -226,6 +229,9 @@ enum gf_internal_fop_indicator {
 #define VIRTUAL_QUOTA_XATTR_CLEANUP_KEY "glusterfs.quota-xattr-cleanup"
 #define QUOTA_READ_ONLY_KEY "trusted.glusterfs.quota.read-only"
 
+/* ctime related */
+#define CTIME_MDATA_XDATA_KEY "set-ctime-mdata"
+
 /* afr related */
 #define AFR_XATTR_PREFIX "trusted.afr"
 
@@ -270,7 +276,7 @@ enum gf_internal_fop_indicator {
 
 #define GLUSTERFS_RPC_REPLY_SIZE 24
 
-#define STARTING_EVENT_THREADS 1
+#define STARTING_EVENT_THREADS 2
 
 #define DEFAULT_VAR_RUN_DIRECTORY DATADIR "/run/gluster"
 #define DEFAULT_GLUSTERFSD_MISC_DIRETORY DATADIR "/lib/misc/glusterfsd"
@@ -284,7 +290,7 @@ enum gf_internal_fop_indicator {
 /* GlusterFS's maximum supported Auxiliary GIDs */
 #define GF_MAX_AUX_GROUPS 65535
 
-#define GF_UUID_BUF_SIZE 50
+#define GF_UUID_BUF_SIZE 37          /* UUID_CANONICAL_FORM_LEN + NULL */
 #define GF_UUID_BNAME_BUF_SIZE (320) /* (64 + 256) */
 
 #define GF_REBALANCE_TID_KEY "rebalance-id"
@@ -325,6 +331,8 @@ enum gf_internal_fop_indicator {
 #define GF_RESPONSE_LINK_COUNT_XDATA "gf_response_link_count"
 #define GF_REQUEST_LINK_COUNT_XDATA "gf_request_link_count"
 
+#define GF_GET_FILE_BLOCK_COUNT "gf_get_file_block_count"
+
 #define CTR_ATTACH_TIER_LOOKUP "ctr_attach_tier_lookup"
 
 #define CLIENT_CMD_CONNECT "trusted.glusterfs.client-connect"
@@ -343,6 +351,8 @@ enum gf_internal_fop_indicator {
 #define GF_LOG_FLUSH_TIMEOUT_MAX_STR "300"
 #define GF_LOG_LOCALTIME_DEFAULT 0
 
+#define GF_NETWORK_TIMEOUT 42
+
 #define GF_BACKTRACE_LEN 4096
 #define GF_BACKTRACE_FRAME_COUNT 7
 
@@ -367,6 +377,10 @@ enum gf_internal_fop_indicator {
 #define GF_CS_OBJECT_STATUS "trusted.glusterfs.cs.status"
 #define GF_CS_OBJECT_REPAIR "trusted.glusterfs.cs.repair"
 
+#define gf_boolean_t bool
+#define _gf_false false
+#define _gf_true true
+
 typedef enum {
     GF_CS_LOCAL = 1,
     GF_CS_REMOTE = 2,
@@ -563,6 +577,10 @@ struct _cmd_args {
     /* FUSE writeback cache support */
     int kernel_writeback_cache;
     uint32_t attr_times_granularity;
+
+    int fuse_flush_handle_interrupt;
+    int fuse_auto_inval;
+    bool brick_mux;
 };
 typedef struct _cmd_args cmd_args_t;
 
@@ -686,6 +704,7 @@ struct _glusterfs_ctx {
     char btbuf[GF_BACKTRACE_LEN];
 
     pthread_mutex_t notify_lock;
+    pthread_mutex_t cleanup_lock;
     pthread_cond_t notify_cond;
     int notifying;
 
@@ -706,6 +725,12 @@ struct _glusterfs_ctx {
     } stats;
 
     struct list_head volfile_list;
+
+    /* Add members to manage janitor threads for cleanup fd */
+    struct list_head janitor_fds;
+    pthread_cond_t janitor_cond;
+    pthread_mutex_t janitor_lock;
+    pthread_t janitor;
 };
 typedef struct _glusterfs_ctx glusterfs_ctx_t;
 
